
#this is the content of codes below:
#Overall effect size
#moderator
#publication bias: Funnel plotting
#sensitivity analysis :quartile-based analysis
#sensitivity analysis : leave-one-out

# Load libraries ----------------------------------------------------------

library(readr)
library(here)
library(metafor)
library(clubSandwich)
library(dplyr)
library(readxl)
library(penalized)

# Load data ---------------------------------------------------------------

effect_sizes <- read_csv(
  here("", "effect_sizes.csv"),#if wrong，open code file first not open R. 
  col_types = cols(
    effect_size_id = col_integer(),
    sample_size = col_integer(),
    professional_level = col_factor(levels = c(
      "Professional vs.Amateur","High-ranked vs. Amateur")),
    cognitive_level = col_factor(
      levels = c("Basic cognition","Higher cognition" ) ),
    cognitive_paradigm = col_factor(
      levels = c(
        "bottom up attention",
        "inhibition",
        "motor control",
        "perception",
        "problem solving",
        "spatial cognition",
        "task switch or multitask",
        "top down attention",
        "verbal cognition"
      ) ),
    DV_type = col_factor(levels = c("accuracy", "speed")),
    gender = col_factor(levels = c("male", "both")),
    game = col_factor(levels = c("Shooting games", "Strategy games","Mix games","Other games")),
    Mean_age = col_double(),
    continent = col_factor(levels = c("EA", "Asia")),
    year = col_integer(),
    female_ratio = col_double()
  )
)

# Overall effect size
overall_model <- rma.mv(
  yi = es,
  V = var,
  slab = reference,
  data = effect_sizes,
  random = ~ 1 | study_id / effect_size_id,
  sparse = TRUE
)
#We use cluster robust standard errors, with small sample size correction from the `clubSandwich` package. The overall effect size, with 95% CI is:
conf_int(overall_model, cluster = effect_sizes$study_id, vcov = "CR2") #CR2:Cluster-Robust estimate
confint(overall_model)

#moderator:professional_level
professional_level_model <- update(
  overall_model,
  mods = ~ 0 + professional_level,
  sparse = TRUE
)
##explain：cluster = effect_sizes$study_id, Which could accumulate the correlation within a study
conf_int(professional_level_model, cluster = effect_sizes$study_id, vcov = "CR2")

coef_test(professional_level_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(p_Satt = p.adjust(p_Satt, "BH"))#BH: Benjamini-Hochberg adjust for p
##
confint(professional_level_model)

##
Wald_test(
  professional_level_model,
  constraints = constrain_equal(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2"
)

##
Wald_test(
  professional_level_model,
  constraints = constrain_pairwise(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2",
  tidy = TRUE
) |> 
  mutate(
    t = sqrt(Fstat),
    hypothesis = gsub("professional_level", "", hypothesis),
    p_val = p.adjust(p_val, "BH")
  )

## Common language effect size

conf_int(professional_level_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(
    across(c("beta", "CI_L", "CI_U"), transf.dtocles),
    ##~round(.x*100))Used to convert effect quantities into a percentage form, 
    ##meaning that experts'cognition have a higher probability than amateurs
    across(c("beta", "CI_L", "CI_U"), ~round(.x*100))
  )

# 2.cognitive_paradigm analysis
cognitive_paradigm_model <- update(
  overall_model,
  mods = ~ 0 + cognitive_paradigm,
  sparse = TRUE
)

conf_int(cognitive_paradigm_model, cluster = effect_sizes$study_id, vcov = "CR2")

coef_test(cognitive_paradigm_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(p_Satt = p.adjust(p_Satt, "BH"))

confint(cognitive_paradigm_model)

Wald_test(
  cognitive_paradigm_model,
  constraints = constrain_equal(1:9),
  cluster = effect_sizes$study_id,
  vcov = "CR2"
)  
Wald_test(
  cognitive_paradigm_model,
  constraints = constrain_pairwise(1:9),
  cluster = effect_sizes$study_id,
  vcov = "CR2",
  tidy = TRUE
) |> 
  mutate(
    t = sqrt(Fstat),
    hypothesis = gsub("cognitive_paradigm", "", hypothesis),
    p_val = p.adjust(p_val, "BH")
  )

##
conf_int(cognitive_paradigm_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(
    across(c("beta", "CI_L", "CI_U"), transf.dtocles),
    across(c("beta", "CI_L", "CI_U"), ~round(.x*100))
  )

#3.DV_type analysis

DV_type_model <- update(
  overall_model,
  mods = ~ 0 + DV_type,
  sparse = TRUE
)

conf_int(DV_type_model, cluster = effect_sizes$study_id, vcov = "CR2")

coef_test(DV_type_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(p_Satt = p.adjust(p_Satt, "BH"))

confint(DV_type_model)

Wald_test(
  DV_type_model,
  constraints = constrain_equal(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2"
)

Wald_test(
  DV_type_model,
  constraints = constrain_pairwise(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2",
  tidy = TRUE
) |> 
  mutate(
    t = sqrt(Fstat),
    hypothesis = gsub("DV_type", "", hypothesis),
    p_val = p.adjust(p_val, "BH")
  )

conf_int(DV_type_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(
    across(c("beta", "CI_L", "CI_U"), transf.dtocles),
    across(c("beta", "CI_L", "CI_U"), ~round(.x*100))
  )

#game genre
game_model <- update(
  overall_model,
  mods = ~ 0 + game,
  sparse = TRUE
)

conf_int(game_model, cluster = effect_sizes$study_id, vcov = "CR2")

coef_test(game_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(p_Satt = p.adjust(p_Satt, "BH"))

confint(game_model)

Wald_test(
  game_model,
  constraints = constrain_equal(c(1,2,3,4)),
  cluster = effect_sizes$study_id,
  vcov = "CR2"
)

Wald_test(
  game_model,
  constraints = constrain_pairwise(c(1,2,3)),
  cluster = effect_sizes$study_id,
  vcov = "CR2",
  tidy = TRUE
) |> 
  mutate(
    t = sqrt(Fstat),
    hypothesis = gsub("game", "", hypothesis),
    p_val = p.adjust(p_val, "BH")
  )

conf_int(game_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(
    across(c("beta", "CI_L", "CI_U"), transf.dtocles),
    across(c("beta", "CI_L", "CI_U"), ~round(.x*100))
  )

# 5.gender analysis
gender_model <- update(
  overall_model,
  mods = ~ 0 + gender,
  sparse = TRUE
)

conf_int(gender_model, cluster = effect_sizes$study_id, vcov = "CR2")

coef_test(gender_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(p_Satt = p.adjust(p_Satt, "BH"))

confint(gender_model)

Wald_test(
  gender_model,
  constraints = constrain_equal(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2"
)

Wald_test(
  gender_model,
  constraints = constrain_pairwise(1:2),
  cluster = effect_sizes$study_id,
  vcov = "CR2",
  tidy = TRUE
) |> 
  mutate(
    t = sqrt(Fstat),
    hypothesis = gsub("gender", "", hypothesis),
    p_val = p.adjust(p_val, "BH")
  )

conf_int(gender_model, cluster = effect_sizes$study_id, vcov = "CR2") |> 
  as.data.frame() |> 
  mutate(
    across(c("beta", "CI_L", "CI_U"), transf.dtocles),
    across(c("beta", "CI_L", "CI_U"), ~round(.x*100))
  )

#6. age analysis

age_model <- update(
  overall_model,
  mods = ~ 1 + Mean_age,
  sparse = TRUE
)
summary(age_model)
coef_test(age_model, vcov = "CR2")

# 6.female_ratio analysis

female_ratio_model <- update(
  overall_model,
  mods = ~ 1 + female_ratio,
  sparse = TRUE
)

## When female_ratio is 0, all are male and the effect size should not be 0, so set the intercept term ~1
summary(female_ratio_model)
coef_test(female_ratio_model, vcov = "CR2")

#model results
summary(overall_model) 
summary(professional_level_model)
summary(cognitive_paradigm_model)
summary(DV_type_model) 
summary(game_model) 
summary(gender_model)
summary(age_model) 



#publication bias: Funnel plotting
funnel(overall_model, yaxis = "seinv", xlim = c(-2.5, 3), ylim = c(1, 5.4), 
       xlab = "Hedge's g", ylab = "Inverse Variance", steps = 5, digits = c(0,1),
       level = c(90, 95, 99), shade = c("white", "gray50", "gray70"),
       refline = c(0), back = "LightBlue", 
       lwd = 1, pch = 20, col = gray(0.25))
es = overall_model$beta
abline(v = c(0,es), col = "black", lty = c(1,2), lwd = c(1,2), pch = 15)
axis(side = 1, at = es, labels = round(es,3), tick = TRUE, col.axis = "black")
rect(2.0,4.1,2.8,5,col = "white", border = "black")
legend(2,5,c("p > 0.1","0.1 > p > 0.05","0.05 > p > 0.01","p < 0.01"),
       fill = c("white", "gray50","gray70","LightBlue"), cex = 0.6, text.width = 1, text.font = 4, bty = "n")


#sensitivity analysis :quartile-based analysis
#Removing extreme values from the model

```{r}
effect_sizes <- read_csv(
  here("file path", "effect_sizes.csv"))

# product overall_model
overall_model <- rma.mv(
  yi = es,
  V = var,
  slab = reference,
  data = effect_sizes,
  random = ~ 1 | study_id / effect_size_id,
  sparse = TRUE
)

# Removal of extreme values more than 3 quartiles apart
effect_sizes <- read_csv(
  here("file path", "effect_sizes.csv"))

# product overall_model
overall_model <- rma.mv(
  yi = es,
  V = var,
  slab = reference,
  data = effect_sizes,
  random = ~ 1 | study_id / effect_size_id,
  sparse = TRUE
)

# Removal of extreme values more than 3 quartiles apart
Q1 <- quantile(effect_sizes$es, 0.25, na.rm = TRUE)
Q3 <- quantile(effect_sizes$es, 0.75, na.rm = TRUE)
IQR <- Q3 - Q1
sub_data <- effect_sizes[effect_sizes$es >= Q1 - 1.5 * IQR & effect_sizes$es <= Q3 + 1.5 * IQR, ]

# Meta-analysis of new data
sub_model <- rma.mv(
  yi = sub_data$es,
  V = sub_data$var,
  slab = sub_data$reference,
  data = sub_data,
  random = ~ 1 | study_id / effect_size_id,
  sparse = TRUE    
)

# Calculate the difference between the new meta-analysis results and the full dataset
sub_diff <- abs(sub_model$beta - overall_model$beta)

# Standard errors for calculating variances
sub_diff_se <- sqrt(sub_model$se^2 + overall_model$se^2)

# Calculate the z-value of the difference
sub_diff_z <- sub_diff / sub_diff_se

# Calculate the p-value
sub_diff_p <- 2 * (1 - pnorm(abs(sub_diff_z)))

# Combine results into one dataframe
# see results in the "enviornment"
sub_summary <- data.frame(
  δ = sub_diff,
  g = sub_model$beta,
  se = sub_diff_se,
  z = sub_diff_z,
  p = sub_diff_p
)

#sensitivity analysis : leave-one-out

# rma.mv overall_model
overall_model <- rma.mv(
  yi = es,
  V = var,
  slab = reference,
  data = effect_sizes,
  random = ~ 1 | study_id / effect_size_id,
  sparse = TRUE
)

# Get the list of student_ids in the dataset
study_list <- unique(effect_sizes$study_id)
study_lab <- unique(effect_sizes$reference)

# Create an empty list of effector differences and p-values
diff_results <- list()
p_results <- list()

# Iterate through each study
for (study in study_list) {
  
  # Create a subdataset that removes the current study
  leave_data <- effect_sizes[effect_sizes$study_id != study,]
  
  # Rerun the rma.mv function using the subdataset
  leave_model <- rma.mv(
    yi = leave_data$es,
    V = leave_data$var,
    slab = leave_data$reference,
    data = leave_data,
    random = ~ 1 | study_id / effect_size_id,
    sparse = TRUE
  )
  
  
  # Calculate the difference between the results of each meta-analysis and the full dataset
  leave_diff <- abs(leave_model$beta - overall_model$beta)
  
  # Standard errors for calculating variances
  leave_diff_se <- sqrt(leave_model$se^2 + overall_model$se^2)
  
  # z
  leave_diff_z <- leave_diff / leave_diff_se
  
  # p
  leave_diff_p <- 2 * (1 - pnorm(abs(leave_diff_z)))
  
  # Adding results to a list
  diff_results <- c(diff_results, leave_diff)
  p_results <- c(p_results, leave_diff_p)
}

# Combine diff_results and p_results lists into one dataframe
leave_summary <- data.frame(
  study_num = study_list,
  study_lab,
  Leave_Diff = unlist(diff_results),
  Leave_Diff_P = unlist(p_results)
)
write.table(leave_summary, file = "leave_one_out.csv", sep = ",", row.names = FALSE)